package com.fjhx.purchase.service.purchase.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.extra.spring.SpringUtil;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.common.constant.SourceConstant;
import com.fjhx.common.entity.documentary.bo.DocumentaryData;
import com.fjhx.common.service.documentary.GetDocumentaryBusinessTemplate;
import com.fjhx.customer.service.customer.CustomerService;
import com.fjhx.item.entity.product.vo.ProductInfoVo;
import com.fjhx.item.service.product.ProductInfoService;
import com.fjhx.purchase.entity.arrival.po.Arrival;
import com.fjhx.purchase.entity.invoice.vo.InvoiceDetailsVo;
import com.fjhx.purchase.entity.pay.vo.PayDetailVo;
import com.fjhx.purchase.entity.purchase.bo.PurchaseDocumentaryBo;
import com.fjhx.purchase.entity.purchase.dto.PurchaseDto;
import com.fjhx.purchase.entity.purchase.dto.PurchaseSelectDto;
import com.fjhx.purchase.entity.purchase.enums.PurchaseStatusEnum;
import com.fjhx.purchase.entity.purchase.po.Purchase;
import com.fjhx.purchase.entity.purchase.po.PurchaseDetail;
import com.fjhx.purchase.entity.purchase.vo.PurchaseDocumentaryVo;
import com.fjhx.purchase.entity.purchase.vo.PurchaseVo;
import com.fjhx.purchase.mapper.purchase.PurchaseMapper;
import com.fjhx.purchase.service.arrival.ArrivalDetailService;
import com.fjhx.purchase.service.arrival.ArrivalService;
import com.fjhx.purchase.service.invoice.InvoiceDetailsService;
import com.fjhx.purchase.service.pay.PayDetailService;
import com.fjhx.purchase.service.purchase.PurchaseDetailService;
import com.fjhx.purchase.service.purchase.PurchaseService;
import com.fjhx.purchase.util.code.CodeEnum;
import com.fjhx.supply.entity.supplier.po.SupplierInfo;
import com.fjhx.supply.service.supplier.SupplierInfoService;
import com.fjhx.wms.entity.warehouse.po.Warehouse;
import com.fjhx.wms.service.warehouse.WarehouseService;
import com.obs.services.internal.ServiceException;
import com.ruoyi.common.core.domain.BaseSelectDto;
import com.ruoyi.common.utils.PageUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.wrapper.IWrapper;
import org.apache.commons.collections4.MapUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * <p>
 * 采购 服务实现类
 * </p>
 *
 * @author
 * @since 2023-04-06
 */
@DS(SourceConstant.PURCHASE)
@Service
public class PurchaseServiceImpl extends ServiceImpl<PurchaseMapper, Purchase>
        implements PurchaseService, GetDocumentaryBusinessTemplate {

    @Autowired
    private SupplierInfoService supplierInfoService;

    @Autowired
    private InvoiceDetailsService invoiceDetailsService;

    @Autowired
    private PayDetailService payDetailService;

    @Autowired
    private PurchaseDetailService purchaseDetailService;

    @Autowired
    private ProductInfoService productInfoService;

    @Autowired
    private ArrivalDetailService arrivalDetailService;

    @Autowired
    private CustomerService customerService;
    @Autowired
    WarehouseService warehouseService;
    @Autowired
    ArrivalService arrivalService;

    /**
     * 列表
     *
     * @param dto
     * @return
     */
    @Override
    public Page<PurchaseVo> getPage(PurchaseSelectDto dto) {
        IWrapper<Purchase> wrapper = getWrapper();
        wrapper.orderByDesc("p", Purchase::getCreateTime);
        if (ObjectUtils.isNotEmpty(dto.getPurchaseStatus())) {
            wrapper.eq("p", Purchase::getPurchaseStatus, dto.getPurchaseStatus());
        }
        if (ObjectUtils.isNotEmpty(dto.getArrivalStatus())) {
            wrapper.eq("p", Purchase::getArrivalStatus, dto.getArrivalStatus());
        }
        if (ObjectUtils.isNotEmpty(dto.getPayStatus())) {
            wrapper.eq("p", Purchase::getPayStatus, dto.getPayStatus());
        }
        wrapper.eq("p", Purchase::getCode, dto.getKeyword());//根据采购单号关键字搜索
        Page<PurchaseVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<PurchaseVo> list = page.getRecords();
        if (CollectionUtils.isNotEmpty(list)) {
            List<Long> supplyIds = list.stream().map(PurchaseVo::getSupplyId).collect(Collectors.toList());
            List<SupplierInfo> supplierInfoList = supplierInfoService.list(Wrappers.<SupplierInfo>query().lambda().in(SupplierInfo::getId, supplyIds));
            Map<Long, List<SupplierInfo>> supplyMap = supplierInfoList.stream().distinct().collect(Collectors.groupingBy(SupplierInfo::getId));
            if (MapUtils.isNotEmpty(supplyMap)) {
                for (PurchaseVo p : list) {
                    List<SupplierInfo> supplys = supplyMap.getOrDefault(p.getSupplyId(), null);
                    p.setSupplyName(supplys == null ? null : supplys.get(0).getName());
                }
            }
        }
        return page;
    }

    @Override
    public Page<PurchaseVo> getPageByWdly(PurchaseSelectDto dto) {
        Page<PurchaseVo> page = getPage(dto);
        List<PurchaseVo> list = page.getRecords();
        if (CollectionUtils.isNotEmpty(list)) {
            //维多利亚 赋值收货仓库名称
            List<Long> wids = new ArrayList<>();
            for (PurchaseVo purchaseVo : list) {
                String victoriatouristJson = purchaseVo.getVictoriatouristJson();
                if (ObjectUtils.isNotEmpty(victoriatouristJson)) {
                    JSONObject json = JSONObject.parseObject(victoriatouristJson);
                    wids.add(json.getLong("receiptWarehouseId"));
                }
            }
            if (ObjectUtils.isEmpty(wids)) {
                return page;
            }
            List<Warehouse> warehouses = warehouseService.listByIds(wids);
            Map<Long, String> warehousesMap = warehouses.stream().collect(Collectors.toMap(Warehouse::getId, Warehouse::getName));
            for (PurchaseVo purchaseVo : list) {
                String victoriatouristJson = purchaseVo.getVictoriatouristJson();
                if (ObjectUtils.isNotEmpty(victoriatouristJson)) {
                    JSONObject json = JSONObject.parseObject(victoriatouristJson);
                    String warehouseName = warehousesMap.get(json.getLong("receiptWarehouseId"));
                    json.put("receiptWarehouseName", warehouseName);
                    purchaseVo.setVictoriatouristJson(json.toJSONString());
                }
            }
            //赋值到货状态
//            List<Long> pids = list.stream().map(PurchaseVo::getId).collect(Collectors.toList());
//            List<Arrival> arrivalList = arrivalService.list(q -> q.in(Arrival::getPurchaseId, pids));
//            Map<Long, List<Arrival>> arrivalMap = arrivalList.stream().collect(Collectors.groupingBy(Arrival::getPurchaseId));
//            for (PurchaseVo purchaseVo : list) {
//                List<Arrival> list1 = arrivalMap.get(purchaseVo.getId());
//                if(ObjectUtils.isEmpty(list1)){
//                    purchaseVo.setArrivalStatus(0);
//                    continue;
//                }
//                List<Arrival> arrivalCount = list1.stream().filter(q -> q.equals(1)).collect(Collectors.toList());
//                if(arrivalCount.size()==0){
//                    //未到货
//                    purchaseVo.setArrivalStatus(0);
//                }else if(arrivalCount.size()<=list1.size()){
//                    //部分到货
//                    purchaseVo.setArrivalStatus(1);
//                }else{
//                    //完全到货
//                    purchaseVo.setArrivalStatus(2);
//                }
//            }
        }
        return page;
    }

    /**
     * 详情
     *
     * @param id
     * @return
     */
    @Override
    public PurchaseVo detail(Long id) {
        Purchase purchase = this.getById(id);
        if (ObjectUtils.isEmpty(purchase)) {
            throw new ServiceException("采购单不存在");
        }
        //查询出供应商名称
        SupplierInfo info = supplierInfoService.getById(purchase.getSupplyId());
        PurchaseVo result = BeanUtil.toBean(purchase, PurchaseVo.class);
        result.setSupplyName(info.getName());
        //查询出采购明细列表
        List<PurchaseDetail> purchaseDetailList = purchaseDetailService.
                list(Wrappers.<PurchaseDetail>query().lambda().eq(PurchaseDetail::getPurchaseId, id));
        //查询出产品
        if (CollectionUtils.isNotEmpty(purchaseDetailList)) {
            List<Long> productIds = purchaseDetailList.stream().map(PurchaseDetail::getBussinessId).collect(Collectors.toList());
            List<ProductInfoVo> productInfos = productInfoService.getListByProductIds(productIds);
            Map<Long, List<ProductInfoVo>> productMap = productInfos.stream().distinct().collect(Collectors.groupingBy(ProductInfoVo::getId));
            if (MapUtils.isNotEmpty(productMap)) {
                for (PurchaseDetail p : purchaseDetailList) {
                    //查询出到货总量
                    BigDecimal sumArrivalCount = arrivalDetailService.getSumCount(p.getId(), p.getBussinessId());
                    p.setSumArrivalCount(sumArrivalCount);
                    List<ProductInfoVo> productInfoVos = productMap.getOrDefault(p.getBussinessId(), null);
                    ProductInfoVo vo = productInfoVos.get(0);
                    p.setProductCode(vo.getCode());
                    p.setProductName(vo.getName());
                    p.setProductType(vo.getType());
                    p.setProductCategory(vo.getClassifyName());
                    p.setProductUnit(vo.getUnit());
                    p.setProductSpec(vo.getSpec());
                }
            }
        }
        result.setPurchaseDetailList(purchaseDetailList);
        return result;
    }

    /**
     * 采购新增
     *
     * @param purchase
     */
    @Override
    public void add(Purchase purchase) {
        PurchaseService purchaseService = SpringUtil.getBean(PurchaseService.class);
        PurchaseDetailService purchaseDetailService = SpringUtil.getBean(PurchaseDetailService.class);
        purchase.setCode(CodeEnum.PURCHASE.getCode());
        purchase.setPurchaseStatus(PurchaseStatusEnum.UNDER_REVIEW.getKey());
        purchaseService.save(purchase);
        if (CollectionUtils.isNotEmpty(purchase.getPurchaseDetailList())) {
            for (PurchaseDetail s : purchase.getPurchaseDetailList()) {
                s.setPurchaseId(purchase.getId());
            }
            purchaseDetailService.saveBatch(purchase.getPurchaseDetailList());
        }
    }

    @Override
    public void edit(PurchaseDto purchaseDto) {
        this.updateById(purchaseDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    /**
     * 根据供应商查询采购合同
     *
     * @param supplyId
     * @return
     */
    @Override
    public List<Purchase> getListBySupplyId(String supplyId) {
        if (StringUtils.isEmpty(supplyId)) {
            throw new ServiceException("供应商ID不能为空");
        }
        List<Purchase> list = this.list(Wrappers.<Purchase>query().lambda().eq(Purchase::getSupplyId, supplyId).eq(Purchase::getPurchaseStatus, PurchaseStatusEnum.PASS.getKey()));
        List<Long> ids = list.stream().distinct().map(Purchase::getId).collect(Collectors.toList());
        List<InvoiceDetailsVo> invoiceDetailsList = invoiceDetailsService.getSumMoneyByPurchaseIds(ids);
        Map<Long, BigDecimal> invoiceMap = invoiceDetailsList.stream().collect(Collectors.toMap(InvoiceDetailsVo::getPurchaseId, InvoiceDetailsVo::getSumMoney));
        List<PayDetailVo> payDetailVoList = payDetailService.getSumMoneyByPurchaseIds(ids);
        Map<Long, BigDecimal> payMap = payDetailVoList.stream().collect(Collectors.toMap(PayDetailVo::getPurchaseId, PayDetailVo::getSumMoney));
        for (Purchase p : list) {
            if (MapUtils.isNotEmpty(invoiceMap)) {
                p.setSumInvoiceMoney(invoiceMap.getOrDefault(p.getId(), null));
            } else {
                p.setSumInvoiceMoney(BigDecimal.ZERO);
            }
            if (MapUtils.isNotEmpty(payMap)) {
                p.setSumPayMoney(payMap.getOrDefault(p.getId(), null));
            } else {
                p.setSumPayMoney(BigDecimal.ZERO);
            }
        }

        return list;
    }

    @Override
    public Page<? extends DocumentaryData> getDocumentaryPage(JSONObject selectData, BaseSelectDto dto, List<Long> excludeBusinessId) {
        PurchaseDocumentaryBo bo = selectData.toJavaObject(PurchaseDocumentaryBo.class);

        IWrapper<Purchase> wrapper = IWrapper.getWrapper();
        wrapper.like(Purchase::getCode, bo.getCode());
        wrapper.eq(Purchase::getCreateUser, bo.getUserId());
        wrapper.eq(Purchase::getPurchaseStatus, 30);
        wrapper.notIn(Purchase::getId, excludeBusinessId);
        wrapper.orderByDesc(Purchase::getId);

        Page<Purchase> itemPage = page(dto.getPage(), wrapper);
        Page<PurchaseDocumentaryVo> page = PageUtils.copyPage(itemPage, PurchaseDocumentaryVo.class);
        List<PurchaseDocumentaryVo> records = page.getRecords();

        if (records.size() == 0) {
            return page;
        }

        // 供应商名称
        supplierInfoService.attributeAssign(records, Purchase::getSupplyId, (item, supplier) -> {
            item.setSupplyName(supplier.getName());
        });

        return page;
    }
}
