package com.fjhx.sale.entity.contract.po;

import com.baomidou.mybatisplus.annotation.FieldFill;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.annotation.Version;
import com.ruoyi.common.core.domain.BasePo;
import lombok.Getter;
import lombok.Setter;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 外销合同表
 * </p>
 *
 * @author
 * @since 2023-04-17
 */
@Getter
@Setter
@TableName("contract")
public class Contract extends BasePo {

    /**
     * 合同类型(1外销 2内销)
     */
    private Integer contractType;

    /**
     * 编码
     */
    private String code;

    /**
     * 合同模版ID
     */
    private Long contractTemplateId;

    /**
     * 卖方公司ID
     *
     * <p>
     * {@link com.fjhx.common.entity.contract.po.ContractTemplate}
     */
    private Long sellCorporationId;

    /**
     * 卖方联系人名称
     */
    private String sellContactName;

    /**
     * 卖方联系人电话
     */
    private String sellContactNumber;

    /**
     * 合同审批状态 0草稿 10、审批中；20、驳回  30、审批通过   99终止
     */
    private Integer status;

    /**
     * 到款状态 0未到款  10部分到款  20已到款
     */
    private Integer refundStatus;

    /**
     * 合同状态
     */
    private Integer contractStatus;

    /**
     * 卖方国家表id
     */
    private String sellCountryId;

    /**
     * 卖方省份id
     */
    private String sellProvinceId;

    /**
     * 卖方城市id
     */
    private String sellCityId;

    /**
     * 卖方详细地址
     */
    private String sellAddress;

    /**
     * 买方公司ID
     *
     * <p>
     * {@link com.fjhx.customer.entity.customer.po.Customer}
     */
    private Long buyCorporationId;

    /**
     * 买方联系人名称
     */
    private String buyContactName;

    /**
     * 买方联系人电话
     */
    private String buyContactNumber;

    /**
     * 买方国家表id
     */
    private String buyCountryId;

    /**
     * 买方省份id
     */
    private String buyProvinceId;

    /**
     * 买方城市id
     */
    private String buyCityId;

    /**
     * 买方详细地址
     */
    private String buyAddress;

    /**
     * 买方邮编
     */
    private String buyPostalCode;

    /**
     * 币种（字典account_currency）
     */
    private String currency;

    /**
     * 合同金额
     */
    private BigDecimal amount;

    /**
     * 付款方式（字典funds_payment_method
     */
    private String paymentMethod;

    /**
     * 贸易方式（字典trade_methods）
     */
    private String tradeMethods;

    /**
     * 预付比例
     */
    private String advanceRatio;

    /**
     * 运输方式（字典  shipping_method）
     */
    private String transportMethod;

    /**
     * 运输备注
     */
    private String transportRemark;

    /**
     * 收款账号ID
     *
     * <p>
     * {@link com.fjhx.account.entity.account.po.AccountManagement}
     */
    private Long shroffAccountId;

    /**
     * 审批通过时间
     */
    private Date approvedDate;

    /**
     * 质保期
     */
    private Integer warranty;

    /**
     * 付款条件
     */
    private String remark;

    /**
     * 创建人名称
     */
    private String userName;

    /**
     * 结算状态 0未结算 1已结算
     */
    private Integer settlementStatus;

    /**
     * 交期
     */
    private String deliveryTime;

    /**
     * 汇率json
     * List<ContractCurrencyRate> 转 json
     * {@link com.fjhx.sale.entity.contract.bo.ContractCurrencyRate}
     */
    private String currencyRateJson;

    /**
     * 版本号
     */
    @TableField(fill = FieldFill.INSERT)
    @Version
    private Integer version;

    /**
     * 合同产品列表
     */
    @TableField(exist = false)
    private List<ContractProduct> contractProductList;

    /**
     * 合同出货列表
     */
    @TableField(exist = false)
    private List<ContractShipment> contractShipmentList;

    /**
     * 合同收费项目
     */
    @TableField(exist = false)
    private List<ContractProject> contractProjectList;

}
