package com.fjhx.sale.service.sale.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.dynamic.datasource.toolkit.DynamicDataSourceContextHolder;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.fjhx.account.controller.utils.DateUtils;
import com.fjhx.account.entity.account.vo.AccountPaymentVo;
import com.fjhx.area.utils.AreaUtil;
import com.fjhx.common.constant.SourceConstant;
import com.fjhx.customer.entity.customer.dto.CustomerDto;
import com.fjhx.customer.entity.customer.dto.CustomerSelectDto;
import com.fjhx.sale.entity.contract.po.Contract;
import com.fjhx.sale.entity.contract.vo.ContractPdfInfoVo;
import com.fjhx.sale.entity.contract.vo.ContractVo;
import com.fjhx.sale.entity.quotation.po.QuotationPay;
import com.fjhx.sale.entity.quotation.po.QuotationProduct;
import com.fjhx.sale.entity.sale.po.SaleQuotation;
import com.fjhx.sale.mapper.sale.SaleQuotationMapper;
import com.fjhx.sale.service.contract.ContractService;
import com.fjhx.sale.service.quotation.QuotationPayService;
import com.fjhx.sale.service.quotation.QuotationProductService;
import com.fjhx.sale.service.sale.SaleQuotationService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.sale.util.code.CodeEnum;
import com.obs.services.internal.ServiceException;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.system.utils.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.sale.entity.sale.vo.SaleQuotationVo;
import com.fjhx.sale.entity.sale.dto.SaleQuotationSelectDto;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.fjhx.sale.entity.sale.dto.SaleQuotationDto;
import cn.hutool.core.bean.BeanUtil;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;


/**
 * <p>
 * 报价表 服务实现类
 * </p>
 *
 * @author
 * @since 2023-04-14
 */
@Service
public class SaleQuotationServiceImpl extends ServiceImpl<SaleQuotationMapper, SaleQuotation> implements SaleQuotationService {
    @Autowired
    private QuotationProductService quotationProductService;

    @Autowired
    private QuotationPayService quotationPayService;

    @Autowired
    private ContractService contractService;


    /**
     * 报价表分页
     */
    @Override
    public Page<SaleQuotationVo> getPage(SaleQuotationSelectDto dto) {
        LambdaQueryWrapper<SaleQuotation> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(ObjectUtil.isNotEmpty(dto.getSellCorporationId()),
                SaleQuotation::getSellCorporationId, dto.getSellCorporationId());
        wrapper.eq(ObjectUtil.isNotEmpty(dto.getStatus()), SaleQuotation::getStatus, dto.getStatus());
        wrapper.orderByDesc(SaleQuotation::getCreateTime);
        Page<SaleQuotationVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<SaleQuotationVo> saleQuotationVoList = page.getRecords();

        //赋值用户名称
        UserUtil.assignmentNickName(saleQuotationVoList, BasePo::getCreateUser, SaleQuotationVo::setUserName);
        return page;
    }

    /**
     * 报价表明细
     */
    @Override
    public SaleQuotationVo detail(Long id) {
        //查询报价表详情
        SaleQuotation SaleQuotation = this.getById(id);
        SaleQuotationVo result = BeanUtil.toBean(SaleQuotation, SaleQuotationVo.class);
        if (ObjectUtil.isNotEmpty(result)) {
            //赋值买方公司地址
            setArea(result);
            //查询报价-商品表的详情
            List<QuotationProduct> quotationProductList = quotationProductService.list(Wrappers.<QuotationProduct>lambdaQuery()
                    .eq(QuotationProduct::getSaleQuotationId, result.getId()));
            result.setQuotationProductList(quotationProductList);

            //查询报价-收费项目表的详情
            List<QuotationPay> quotationPayList = quotationPayService.list(Wrappers.<QuotationPay>lambdaQuery()
                    .eq(QuotationPay::getSaleQuotationId, result.getId()));
            result.setQuotationPayList(quotationPayList);
        }

        return result;
    }

    @Override
    public void add(SaleQuotationDto saleQuotationDto) {
        this.save(saleQuotationDto);
    }

    @Override
    public void edit(SaleQuotationDto saleQuotationDto) {
        this.updateById(saleQuotationDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    /**
     * 报价单的复制功能（报价单，报价-商品表的 ，报价-收费项目表，根据报价表的ID相关数据复制一份）
     */
    @Override
    public void copy(SaleQuotationDto saleQuotationDto) {
        if (ObjectUtil.isEmpty(saleQuotationDto.getId())) {
            throw new ServiceException("参数缺失：报价单ID不能为null");
        }
        //查询商品-报价表的数据
        SaleQuotation saleQuotation = this.getById(saleQuotationDto.getId());
        if (ObjectUtil.isNotEmpty(saleQuotation)){
            throw new ServiceException("报价表的数据不存在");
        }

        //查询报价-商品表的数据
        List<QuotationProduct> quotationProductList = quotationProductService.list(Wrappers.<QuotationProduct>lambdaQuery()
                .eq(QuotationProduct::getSaleQuotationId, saleQuotationDto.getId()));

        //查询报价-收费项目表
        List<QuotationPay> quotationPays = quotationPayService.list(Wrappers.<QuotationPay>lambdaQuery()
                .eq(QuotationPay::getSaleQuotationId, saleQuotationDto.getId()));

        //添加商品-报价表
        quotationProductList.forEach(quotationProduct -> quotationProduct.setId(null));
        quotationProductService.saveBatch(quotationProductList);

        //添加报价-收费项目表
        quotationPays.forEach(quotationPay -> quotationPay.setId(null));
        quotationPayService.saveBatch(quotationPays);


        //添加报价表的信息
        saleQuotation.setId(null);
        saleQuotation.setCode(CodeEnum.SALE_QUOTATION.getCode());
        baseMapper.insert(saleQuotation);

    }

    /**
     * 查询这个客户的报价次数
     * @param id （客户ID）
     */
    @Override
    public Integer getCount(Long id) {
      return  baseMapper.getCount(id);
    }

    /**
     * 查询指定客户的每月报价总额
     * @param dto
     * @return
     */
    @Override
    public List<SaleQuotationVo> getAmount(CustomerDto dto) {
        return baseMapper.getAmount(dto);
    }

    /**
     * 最新跟进（客户画像）
     */
    @Override
    public List<Map<String,Object>> latestFollowUp(CustomerDto dto) {

//        //切换数据源
//        DynamicDataSourceContextHolder.push(SourceConstant.SALE);
//        //查询这个客户的报价列表
//        saleQuotationService.page(,Wrappers.<SaleQuotation>lambdaQuery()
//                .eq(SaleQuotation::getBuyCorporationId,dto.getId()).orderByDesc(SaleQuotation::getCreateTime));
//
//        //查询客户的合同列表
//        contractService.list();
        return null;
    }

    /**
     * 销售统计（客户画像）
     */
    @Override
    public Map<String, Object> salesStatistics(CustomerDto dto) {
        Map<String,Object> map = new HashMap<>();
        //切换数据源
        DynamicDataSourceContextHolder.push(SourceConstant.SALE);
        //查询这个客户的销售额（合同总金额）及成交合同数量
        ContractVo contractVo = contractService.getSalesTotal(dto.getId());
        map.put("contractVo",contractVo);

        //查询这个客户的报价次数
        Integer count = this.getCount(dto.getId());
        DynamicDataSourceContextHolder.poll();
        map.put("count",count);
        return map;
    }
    /**
     * 报价销售走势（客户画像）
     */
    @Override
    public Map<String, Object> salesTrend(CustomerDto dto) {
        Map<String, Object> map = new HashMap<>();

        List<String> monthBetweenDate = DateUtils.getMonthBetweenDate(DateUtil.offsetMonth(new Date(), -12), new Date());

        //存放报价的数据；
        List<Map<String,Object>> saleQuotationList = new ArrayList<>();
        //存放报价的数据；
        List<Map<String,Object>> contractList = new ArrayList<>();

        dto.setBeginTime(DateUtil.format(DateUtil.offsetMonth(new Date(), -12),"yy-MM"));
        dto.setEndTime(DateUtil.format(new Date(),"yy-MM"));
        //查询指定客户的每月报价总额
        List<SaleQuotationVo> saleQuotationVoList = this.getAmount(dto);

        Map<String, List<SaleQuotationVo>> saleQuotationVoMap = saleQuotationVoList.stream()
                .collect(Collectors.groupingBy(item->DateUtil.format(item.getCreateTime(),"yyyy-MM")));

        //查询指定客户的每月合同总额
        List<ContractVo> contractVoList = contractService.getAmount(dto);
        Map<String, List<ContractVo>> contractVoMap = contractVoList.stream()
                .collect(Collectors.groupingBy(item->DateUtil.format(item.getCreateTime(),"yyyy-MM")));

        //赋值每个月的值
        for (String month : monthBetweenDate) {
            //存放每月的报价总额
            Map<String,Object> saleQuotationMap = new HashMap<>();
            //每月的金额初始化的值为0
            saleQuotationMap.put("month",month);
            saleQuotationMap.put("saleAmount",new BigDecimal(0));
            List<SaleQuotationVo> saleQuotationVos = saleQuotationVoMap.get(month);
            //给每月的报价金额赋值
            if (ObjectUtil.isNotEmpty(saleQuotationVos)){
                saleQuotationMap.put("saleAmount",saleQuotationVos.get(0).getAmount());
            }
            saleQuotationList.add(saleQuotationMap);

            //存放每月的合同总额
            Map<String,Object> contractMap = new HashMap<>();
            //每月的金额初始化的值为0
            contractMap.put("month",month);
            contractMap.put("contractAmount",new BigDecimal(0));
            List<ContractVo> contractVoList1 = contractVoMap.get(month);
            if (ObjectUtil.isNotEmpty(contractVoList1)){
                contractMap.put("contractAmount",contractVoList1.get(0).getAmount());
            }
            contractList.add(contractMap);
        }
        map.put("contractList",contractList);
        map.put("saleQuotationList",saleQuotationList);
        return map;
    }

    /**
     * 销售明细（客户画像）
     */
    @Override
    public  Page<SaleQuotationVo> saleDetail(CustomerSelectDto dto) {
        Page<SaleQuotationVo> saleQuotationVoPage = baseMapper.saleDetail(dto.getPage(), dto);
        List<SaleQuotationVo> records = saleQuotationVoPage.getRecords();
        // 赋值用户名称
        UserUtil.assignmentNickName(records, SaleQuotationVo::getCreateUser, SaleQuotationVo::setUserName);

        return saleQuotationVoPage;
    }

    /**
     * 赋值买卖公司地址
     */
    private void setArea(SaleQuotationVo saleQuotationVo) {

        List<String> areaIdList = Arrays.asList(
                saleQuotationVo.getBuyCountryId(),
                saleQuotationVo.getBuyProvinceId(),
                saleQuotationVo.getBuyCityId()
        );
        Map<String, String> areaMap = AreaUtil.getAreaMapByIds(areaIdList);
        saleQuotationVo.setBuyCountryName(areaMap.get(saleQuotationVo.getBuyCountryId()));
        saleQuotationVo.setBuyProvinceName(areaMap.get(saleQuotationVo.getBuyProvinceId()));
        saleQuotationVo.setBuyCityName(areaMap.get(saleQuotationVo.getBuyCityId()));
        saleQuotationVo.setBuyDetailedAddress(saleQuotationVo.getBuyAddress());
    }


}
